@@ -14,34 +14,41 @@
 #include "apk_version.h"
 #include "apk_print.h"
 
-static int update_main(void *ctx, struct apk_database *db, struct apk_string_array *args)
+static int update_parse_options(void *ctx, struct apk_ctx *ac, int opt, const char *optarg)
 {
-	struct apk_repository *repo;
-	struct apk_url_print urlp;
-	int i;
-	char buf[32] = "OK:";
+	switch (opt) {
+	case APK_OPTIONS_INIT:
+		ac->cache_max_age = 0;
+		break;
+	default:
+		return -ENOTSUP;
+	}
+	return 0;
+}
 
-	if (apk_verbosity < 1)
+static int update_main(void *ctx, struct apk_ctx *ac, struct apk_string_array *args)
+{
+	struct apk_out *out = &ac->out;
+	struct apk_database *db = ac->db;
+	const char *msg = "OK:";
+	char buf[64];
+
+	if (apk_out_verbosity(out) < 1)
 		return db->repositories.unavailable + db->repositories.stale;
 
-	for (i = 0; i < db->num_repos; i++) {
-		repo = &db->repos[i];
-
-		if (APK_BLOB_IS_NULL(repo->description))
-			continue;
-
-		apk_url_parse(&urlp, db->repos[i].url);
-		apk_message(BLOB_FMT " [" URL_FMT "]",
-			    BLOB_PRINTF(repo->description),
-			    URL_PRINTF(urlp));
+	apk_db_foreach_repository(repo, db) {
+		if (!repo->available) continue;
+		apk_msg(out, BLOB_FMT " [" BLOB_FMT "]",
+			BLOB_PRINTF(repo->description),
+			BLOB_PRINTF(repo->url_printable));
 	}
 
 	if (db->repositories.unavailable || db->repositories.stale)
-		snprintf(buf, sizeof(buf), "%d unavailable, %d stale;",
+		msg = apk_fmts(buf, sizeof buf, "%d unavailable, %d stale;",
 			 db->repositories.unavailable,
-			 db->repositories.stale);
+			 db->repositories.stale) ?: "ERRORS;";
 
-	apk_message("%s %d distinct packages available", buf,
+	apk_msg(out, "%s %d distinct packages available", msg,
 		db->available.packages.num_items);
 
 	return db->repositories.unavailable + db->repositories.stale;
@@ -50,7 +57,7 @@ static int update_main(void *ctx, struct apk_database *db, struct apk_string_arr
 static struct apk_applet apk_update = {
 	.name = "update",
 	.open_flags = APK_OPENF_WRITE | APK_OPENF_ALLOW_ARCH,
-	.update_cache = 1,
+	.parse = update_parse_options,
 	.main = update_main,
 };
 
