@@ -713,6 +713,27 @@ int apk_ipkg_add_script(struct apk_installed_package *ipkg, struct apk_istream *
 	return apk_ipkg_assign_script(ipkg, type, b);
 }
 
+#ifdef __linux__
+static inline int make_device_tree(struct apk_database *db)
+{
+	if (faccessat(db->root_fd, "dev", F_OK, 0) == 0) return 0;
+	if (mkdirat(db->root_fd, "dev", 0755) < 0 ||
+	    mknodat(db->root_fd, "dev/null", S_IFCHR | 0666, makedev(1, 3)) < 0 ||
+	    mknodat(db->root_fd, "dev/zero", S_IFCHR | 0666, makedev(1, 5)) < 0 ||
+	    mknodat(db->root_fd, "dev/random", S_IFCHR | 0666, makedev(1, 8)) < 0 ||
+	    mknodat(db->root_fd, "dev/urandom", S_IFCHR | 0666, makedev(1, 9)) < 0 ||
+	    mknodat(db->root_fd, "dev/console", S_IFCHR | 0600, makedev(5, 1)) < 0)
+		return -1;
+	return 0;
+}
+#else
+static inline int make_device_tree(struct apk_database *db)
+{
+	(void) db;
+	return 0;
+}
+#endif
+
 int apk_ipkg_run_script(struct apk_installed_package *ipkg,
 			struct apk_database *db,
 			unsigned int type, char **argv)
@@ -735,12 +756,13 @@ int apk_ipkg_run_script(struct apk_installed_package *ipkg,
 
 	argv[0] = fn;
 
-	if (db->script_memfd_ok) {
+	if (!db->memfd_failed) {
 		/* Linux kernel >= 6.3 */
 		fd = memfd_create(fn, MFD_EXEC);
 		if (fd < 0 && errno == EINVAL) {
 			/* Linux kernel < 6.3 */
 			fd = memfd_create(fn, 0);
+			if (fd < 0) db->memfd_failed = 1;
 		}
 	}
 	if (!db->script_dirs_checked) {
@@ -748,6 +770,11 @@ int apk_ipkg_run_script(struct apk_installed_package *ipkg,
 			reason = "failed to prepare dirs for hook scripts: ";
 			goto err_errno;
 		}
+		if (!db->root_dev_ok && !db->need_unshare) {
+			if (make_device_tree(db) < 0)
+				apk_warn(out, PKG_VER_FMT ": failed to create initial device nodes: %s",
+					 PKG_VER_PRINTF(pkg), apk_error_str(errno));
+		}
 		db->script_dirs_checked = 1;
 	}
 	if (fd < 0) {
@@ -762,6 +789,10 @@ int apk_ipkg_run_script(struct apk_installed_package *ipkg,
 	if (created) {
 		close(fd);
 		fd = -1;
+	} else {
+#ifdef F_ADD_SEALS
+		fcntl(fd, F_ADD_SEALS, F_SEAL_EXEC);
+#endif
 	}
 
 	apk_msg(out, "%sExecuting " PKG_VER_FMT ".%s",
